from gym.envs.registration import register
import os

SNAP_PATH = os.path.dirname(os.path.realpath(__file__)) + "/snap/"
SNAP_EXT = ".sna"

games = {
    'Arkanoid': 'arka',
    'BuggyBoy': 'buggy'
}

# If you want to add game supported by amle, place the name here
for game in games:

    name_sna = None
    
    # For the snapshot file, favor the format '<gamenamed>.sna' or specify the name like this  
    if(game == 'Arkanoid'):
        name_sna = 'arka'
    if(game == 'BuggyBoy'):
        name_sna = 'buggy'


    game_path = SNAP_PATH + name_sna + SNAP_EXT
    
    # Register Arkanoid game
    register(
        id='{}-v0'.format(game),
        entry_point='gym_cap32.envs:Cap32Env',
        nondeterministic=True,
        kwargs={'game': game, 'game_path':game_path.encode()},
    )



import gym
from gym import error, spaces, utils
from gym.utils import seeding
import os
import numpy as np

try:
    import amle_py
except ImportError as e:
    raise error.DependencyNotInstalled("{}. (HINT: you can install HFO dependencies with 'pip3 install amle-py'.)".format(e))

from amle_py import AmLEInterface


class Cap32Env(gym.Env):
  '''
  This class represent the amle gym environment.
  It's using the amle_python_interface class and it's built by gym.
  '''

  metadata = {'render.modes': ['human']}
  def __init__(self, game, game_path, sound=False, speed=32):
    '''
    Create a new Cap32Env object. The emulator is started with the game loaded.
    The environment is initialized.
    :param game: The game named to loaded.
    :param game_path: The game path to loaded.
    :param sound: Enables/disables sound. Default value: False
    :param speed: Sets the speed of the emulator. Default value: 32
    '''
    self.game = game
    self.game_path = game_path

    if not os.path.exists(self.game_path):
      msg = 'You asked for game %s but path %s does not exist'
      raise IOError(msg % (game, self.game_path))
    
    self.amle = amle_py.AmLEInterface()
    self.amle.toggleSound(sound)
    self.amle.loadSnapshot(game, game_path)

    self.amle.setEmulatorSpeed(speed)

    nbLegalActions = self.amle.getNbLegalActions()
    self._action_set = np.empty(nbLegalActions, dtype=np.int32)
    self.amle.getLegalActions(self._action_set)
    
    self.action_space = spaces.Discrete(len(self._action_set))

    self.screen_height = self.amle.getScreenHeight()
    self.screen_width = self.amle.getScreenWidth()
    self.observation_space = spaces.Box(low=0, high=255, shape=(self.screen_height, self.screen_width), dtype=np.int32)
    
    
  
  def step(self, a):
    '''
    Makes a step in the emulation, goes to the next frame, applies effect of given actions.
    :param a: The action to send.
    :return: Return the observation, reward, a game over boolean and informations as the number of lives.
    '''
    reward = 0.0  
    action = self._action_set[a]
    reward = self.amle.act(action)
    ob = np.zeros(shape=(self.screen_height, self.screen_height))
    self.amle.getScreen(ob)
    return ob, reward, self.amle.gameOver(), {"alme.getNbLives": self.amle.getNbLives()}




  def reset(self):
    '''
    Reset the game.
    '''
    self.amle.resetGame()
    
  def render(self, mode='human'):
    '''
    Empty function because the emulator necessarily display the screen game.
    '''
    pass
  


import setuptools

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name="gym-cap32", # Replace with your own username
    version="1.0.7",
    author="Theo De castro Pinto, Wilfried Augeard",
    author_email="decastrotheo960@gmail.com",
    description="Gym environment for Amstrad CPC.",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    package_data={
        "amle_py": ["snap/*.sna"]
    },
    include_package_data=True,
)
